# -*- coding: utf-8 -*- #
from zephir.utils.creolewrap import ZephirDict, config_get_values, PropertiesOptionError, get_load_errors
from zephir.eolerpclib import xmlrpclib, EoleProxy
from zephir.web import config
from flask import session, current_app, redirect, send_file, url_for
import base64, os, cjson, traceback
import ConfigParser

from zephir.web.zephirgenconfig import app
from eolegenconfig.views import *

def init_zephir(id_, zephir_data, session_data={}, load_error=False, force_load_owner=None):
    lib.init_session(id_)
    source = session_data['source']
    force_instanciate = 'non'
    no_auto_store = False
    if source == "serveur" and session_data.get('registered', 0) == 1:
        force_instanciate = 'oui'
    if source in  ('module', 'variante'):
        no_auto_store = True
    dico = ZephirDict(mode='', version="creole3", force_instanciate=force_instanciate, no_auto_store=no_auto_store)
    dico.init_from_zephir(zephir_data, load_error, force_load_owner=force_load_owner, warnings=True)
    # définition du 'owner' des valeurs éditées :
    # module / variante / serveur(->zephir)
    zephir_owners = {'serveur':'zephir'}
    dico.set_owner(zephir_owners.get(source, source))
    if source == 'serveur':
        # préremplissage de rne et libelle_etab si pas encore saisis
        try:
            if force_instanciate == 'non' and dico.get_var('numero_etab')[1] == ""  and session_data['rne']:
                dico.set_value(session_data['rne'])
            if dico.get_var('libelle_etab')[1] == "" and session_data['libelle_etab']:
                dico.set_value(session_data['libelle_etab'])
        except:
            traceback.print_exc()
            app.logger.error('Erreur au pré-remplissage des informations établissement')
    # stockage de la configuration tiramisu et des données d'origine (sans infos d'upgrade)
    if type(zephir_data[-1]) == dict and 'load_errors' in zephir_data[-1]:
        del(zephir_data[-1])
    session_data.update({'config':dico.dico, 'zephir_data':zephir_data})
    lib.store_session_data(id_, session_data)

# surcharge des fonctions originales d'eolegenconfig
def get_zephir_id():
    # en mode zephir : on utilise le cookie de l'application web Zéphir
    return request.cookies['TWISTED_SESSION']

def get_var_module(zephir_proxy, id_var):
    rc, mod_info = zephir_proxy.modules.get_variante(id_var)
    assert rc == 1, mod_info
    return int(mod_info[0]['module'])


lib.get_id = get_zephir_id
lib.init_zephir = init_zephir


def get_proxy(request):
    # récupération des infos de la requête d'origine transmis par twisted WSGI
    session.pop('username', None)
    assert 'HTTP_AUTHORIZATION' in request.environ, 'Unauthorized access'
    auth_info = request.environ['HTTP_AUTHORIZATION'].split()[1]
    user, passwd = base64.decodestring(auth_info).split(':')
    # XXX zephir_web est prévu pour fonctionner avec un backend local pour un
    # backend distant, passer l'adresse en paramètre et utiliser config.PORT_ZEPHIR
    port_zephir = str(int(config.PORT_ZEPHIR) + 1)
    proxy_addr = "http://{0}:{1}@localhost:{2}/".format(user, passwd, port_zephir)
    # on met à jour le nom de l'utilisateur dans la session flask d'eolegenconfig
    session['username'] = user
    return EoleProxy(proxy_addr)

@app.route('/serveur/<id_serveur>/<edit_mode>')
def init_serveur(id_serveur, edit_mode):
    """Entry point for editing a server configuration through Zephir
    """
    try:
        zephir_proxy = get_proxy(request)
        rne = libelle_etab = ""
        try:
            # récupération rne/libellé établissement du serveur
            rc, data = zephir_proxy.serveurs.get_serveur(int(id_serveur))
            assert rc == 1
            rne = data[0]['rne']
            rc, data = zephir_proxy.etabs.get_etab(rne)
            assert rc == 1
            libelle_etab = data[0]['libelle']
        except:
            traceback.print_exc()
            app.logger.error(u"Erreur de récupération des infos établissement pour le serveur {0}".format(id_serveur))
        rc, status_data = zephir_proxy.serveurs.get_status(int(id_serveur))
        if edit_mode == "modif_config":
            # dans le cas d'un serveur (édition), on détermine si la procédure
            # d'enregistrement a déjà eu lieu (serveur instancié)
            registered = status_data.get('cle_ok', 0)
        else:
            registered = 0
        # récupération des dictionnaires/valeurs
        rc, data = zephir_proxy.serveurs.get_dico(int(id_serveur), edit_mode)
        assert rc == 1, data
        session_data = {'source':'serveur', 'source_id':int(id_serveur), 'proxy':zephir_proxy, \
        'edit_mode':edit_mode, 'rne':rne, 'libelle_etab':libelle_etab, 'registered':registered}
        init_zephir(get_zephir_id(), data, session_data)
    except xmlrpclib.ProtocolError:
        return make_error_response(u"Vous n'êtes pas autorisé à gérer la configuration de ce serveur", 403)
    except Exception, err:
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))
    return redirect(url_for('.root',zephir=1,title="Genconfig: serveur {0}".format(id_serveur)))

@app.route('/migrate/<id_serveur>/<id_variante>/<edit_mode>')
def migrate_serveur(id_serveur, id_variante, edit_mode='migration'):
    """Entry point for editing a server configuration through Zephir
    """
    try:
        zephir_proxy = get_proxy(request)
        # récupération de la configuration migrée avec les paramètres venant de la variante
        rc, data = zephir_proxy.serveurs.migrate_conf(int(id_serveur),edit_mode,int(id_variante))
        assert rc == 1, data
        session_data = {'source':'serveur', 'source_id':int(id_serveur), 'proxy':zephir_proxy, 'edit_mode':edit_mode, 'registered':0}
        init_zephir(get_zephir_id(), data, session_data)
    except xmlrpclib.ProtocolError:
        return make_error_response(u"Vous n'êtes pas autorisé à gérer la configuration de ce serveur", 403)
    except Exception, err:
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))
    return redirect(url_for('.root',zephir=1,title="Genconfig: serveur {0}".format(id_serveur)))

@app.route('/module/<id_module>')
def init_module(id_module):
    """Entry point of the application for editing a module default values through Zephir
    """
    try:
        zephir_proxy = get_proxy(request)
        # récupération des dictionnaires/valeurs
        rc, data = zephir_proxy.modules.get_dico(int(id_module))
        assert rc == 1, data
        session_data = {'source':'module', 'source_id':int(id_module), 'proxy':zephir_proxy}
        init_zephir(get_zephir_id(), data, session_data)
    except xmlrpclib.ProtocolError:
        return make_error_response(u"Vous n'êtes pas autorisé à accéder à ce module", 403)
    except Exception, err:
        import traceback
        traceback.print_exc()
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))
    return redirect(url_for('.root',zephir=1,title="Genconfig: module {0}".format(id_module)))

@app.route('/variante/<id_variante>')
def init_variante(id_variante):
    """Entry point of the application for editing a variante default values through Zephir
    """
    try:
        zephir_proxy = get_proxy(request)
        # récupération des dictionnaires/valeurs
        id_module = get_var_module(zephir_proxy, id_variante)
        rc, data = zephir_proxy.modules.get_dico(id_module, int(id_variante))
        assert rc == 1, data
        session_data = {'source':'variante', 'source_id':int(id_variante), 'proxy':zephir_proxy}
        init_zephir(get_zephir_id(), data, session_data)
    except xmlrpclib.ProtocolError:
        return make_error_response(u"Vous n'êtes pas autorisé à accéder à cette variante", 403)
    except Exception, err:
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))
    return redirect(url_for('.root',zephir=1,title="Genconfig: variante {0}".format(id_variante)))

# redéfinition de l'import de fichier de configuration. Permet de conserver
# la référence aux valeurs d'origine provenant de zéphir
@app.route('/upload', methods=['POST'])
def upload_config():
    """get values from client stored configuration file
    """
    try:
        eol_file = request.files['file']
        if not eol_file or not os.path.splitext(eol_file.filename)[1] == '.eol':
            return make_error_response(u"Extension de fichier non autorisée")
        else:
            # récupération des valeurs du fichier
            load_error = False
            try:
                f_values = cjson.decode(eol_file.stream.read().strip(), all_unicode=True)
            except:
                app.logger.error("Tentative d'import d'un fichier Creole2")
                # essai d'import à l'ancien format
                try:
                    old_conf = ConfigParser.ConfigParser()
                    eol_file.seek(0)
                    old_conf.readfp(eol_file.stream, 'zephir_ini.eol')
                    assert len(old_conf._sections) > 0
                    f_values = old_conf._sections
                    load_error = True
                except Exception, err:
                    app.logger.error(err, exc_info=True)
                    return make_error_response(u"Fichier non compatible ({0})".format(err))
                # erreur, au chargement, fichier de type creole2 ?
            id_ = get_zephir_id()
            # mise à jour des valeurs récupérées de Zéphir et réinitialisation de la configuration
            orig_values = eval(lib.session_tiramisu[id_]['zephir_data'][-1])
            lib.session_tiramisu[id_]['zephir_data'][-1] = str(f_values)
            # on réinitialise la configuration en forçant les valeurs précédentes
            # comme valeurs d'origine (sinon genconfig ne détecte pas de modifications)
            lib.del_config(id_)
            if load_error:
                load_owner = 'zephir_upgrade'
            else:
                load_owner = 'zephir_import'
            config = lib.get_config(id_, load_error=load_error, force_load_creole_owner=load_owner)
            config.impl_set_information('orig_values', orig_values)
            # vérification des valeurs importées
            load_errors = get_load_errors(config)
            for load_error in load_errors.values():
                app.logger.error(unicode(load_error, 'utf-8'))
            version = lib.get_upgrade_version(id_)
            return make_json_response({ 'filename': eol_file.filename, 'errors': load_errors, 'version': version})
    except Exception, err:
        #FIXME mettre dans les logs
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))

# redéfinition pour Zéphir, ajoute type type de ressource et identifiant
# au nom du fichier téléchargé (au lieu de config.eol)
# pas de validation des valeurs obligatoires en mode module/variante
@app.route('/download', methods=['GET'])
def download_config():
    try:
        id_ = lib.get_id()
        session_data = lib.session_tiramisu[id_]
        source = session_data['source']
        source_id = session_data['source_id']
        config = session_data['config']
        if source == 'serveur':
            json_config = config_get_values(config, 'creole')
        else:
            json_config = config_get_values(config, 'creole', check_mandatory=False, ignore_autofreeze=True)
        json_config = cjson.encode(json_config)
        strIO = StringIO.StringIO()
        strIO.write(json_config)
        strIO.seek(0)
        return send_file(strIO,
                     attachment_filename="{0}-{1}.eol".format(source, source_id),
                     as_attachment=True)
    except Exception, err:
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))

@app.route('/save', methods=['POST'])
def save_config():
    """sends configuration values back to zephir
    """
    try:
        id_ = get_zephir_id()
        config = lib.get_config(id_)
        # envoi des valeurs à Zéphir
        source = lib.session_tiramisu[id_]['source']
        source_id = lib.session_tiramisu[id_]['source_id']
        zephir_proxy = lib.session_tiramisu[id_]['proxy']
        zephir_data = lib.session_tiramisu[id_]['zephir_data']
        if source == 'serveur':
            values = config_get_values(config, 'creole')
            edit_mode = lib.session_tiramisu[id_]['edit_mode']
            rc, msg = zephir_proxy.serveurs.save_conf(source_id, [str(values)] , edit_mode)
        elif source == 'module':
            values = config_get_values(config, 'creole',check_mandatory=False, ignore_autofreeze=True)
            rc, msg = zephir_proxy.modules.save_dico([str(values)] , source_id)
        elif source == 'variante':
            values = config_get_values(config, 'creole',check_mandatory=False, ignore_autofreeze=True)
            # cas particulier, on ne sauvegarde pas les valeurs du module dans le fichier de variante
            varnames = values.keys()
            for varname in varnames:
                if values[varname].get('owner','') == 'module':
                    del(values[varname])
            id_module = get_var_module(zephir_proxy, source_id)
            pass_var = "XXX FIXME : mot de passe variante ..."
            rc, msg = zephir_proxy.modules.save_dico([str(values)], id_module, source_id, pass_var)
        # return mode and debug to previous state
        # after reloading config
        mode = lib.get_mode(id_)
        debug = lib.get_debug(id_)
        lib.set_mode(id_, mode)
        lib.set_debug(id_, debug)
        # mise à jour des valeurs modifiées dans zephir_data
        zephir_data[-1] = str(values)
        lib.store_session_data(id_, {'zephir_data':zephir_data})
        lib.del_config(id_)
        return make_json_response({})
    except xmlrpclib.ProtocolError:
        return make_error_response(u"Vous n'êtes pas autorisé à modifier ces données", 403)
    except Exception, err:
        app.logger.error(err, exc_info=True)
        return make_error_response(str(err))

